signature PARSE =
  sig
    type svalue 
    type pos    
    type ('a,'b) token
    type result

    val parse : unit -> result list
    val trees : bool ref
    val debug : bool ref
  end  

functor Parse (structure Interface : INTERFACE
               structure Parser : PARSER
                  sharing type Parser.arg = Interface.arg
                  sharing type Parser.pos = Interface.pos
               structure Tokens : NL__TOKENS
                  sharing type Tokens.token = Parser.Token.token
                  sharing type Tokens.svalue = Parser.svalue
              ) : PARSE =
  struct
    type pos = Parser.pos  
    type svalue = Parser.svalue 
    type ('a,'b) token = ('a,'b) Parser.Token.token
    type result = Parser.result
    val trees = Parser.trees 
    val debug = Parser.debug

    fun parse () =
        let 
          fun reader (n:int) = 
              case (TextIO.inputLine TextIO.stdIn) of
                SOME s => s | NONE => ""
  
          val _ = Interface.init_line()
          val empty = !Interface.line

          val dummyEOF = Tokens.EOF(empty,empty)

          fun loop lexer =
              let 
                  val (result,lexer) = 
                      Parser.parse(lexer,Interface.error,Interface.nothing)
                  val ((nextTokens,_),lexer) = Parser.Stream.get lexer
              in 
                  if Parser.sameToken(hd nextTokens,dummyEOF) 
                  then result else loop lexer
              end
        in
          loop (Parser.makeLexer reader)
        end
  end
structure NL : PARSE =
   Parse (structure Interface = Interface
          structure Parser = NL_Parser
          structure Tokens = NL_Vals.Tokens);

