structure T = Tokens
structure I = Interface
structure A = Absyn

open I

type pos = I.pos
type svalue = T.svalue
type ('a,'b) token = ('a,'b) T.token
type lexresult = (svalue,pos) token list * string

fun eof() = ([T.EOF(!line,!line)],"")

fun PN(s:string) = ([T.PN(A.ConS s,!line,!line)],s)
fun CNn s t = ([T.CNn(A.ConS s,!line,!line)],t)
fun CNg s t = ([T.CNg(A.ConS s,!line,!line)],t)
fun CNd s t = ([T.CNd(A.ConS s,!line,!line)],t)
fun CNa s t = ([T.CNa(A.ConS s,!line,!line)],t)
fun RNn s t = ([T.RNn(A.ConR s,!line,!line)],t)
fun RNg s t = ([T.RNg(A.ConR s,!line,!line)],t)
fun RNd s t = ([T.RNd(A.ConR s,!line,!line)],t)
fun RNa s t = ([T.RNa(A.ConR s,!line,!line)],t)
fun IVs(s:string) = ([T.IVs(A.ConS s,!line,!line)],s)
fun IVp(s:string) = ([T.IVp(A.ConS s,!line,!line)],s)
fun TVs(s:string) = ([T.TVs(A.ConR s,!line,!line)],s)
fun TVp(s:string) = ([T.TVp(A.ConR s,!line,!line)],s)
fun CA(s:string) = ([T.CA(A.ConS s,!line,!line)],s)
fun P(s:string) = ([T.P(A.ConR s,!line,!line)],s)

%%  
%header (functor NL_LexFun(structure Absyn: ABSYN
                           structure Tokens: NL__TOKENS
                              sharing type Absyn.set = Tokens.set 
                           structure Interface: INTERFACE) : LEXER);
%full 

whitespace=[\t\ ]*;
%%
<INITIAL>{whitespace} => (lex());
\n         => (next_line(); lex());
"."        => ([T.PUNKT(!line,!line)],yytext);
"ist"      => ([T.CopVs(!line,!line)],yytext);
"sind"      => ([T.CopVp(!line,!line)],yytext);
"von"    => ([T.von(!line,!line)],yytext);
("der" | "die" | "das")   
           => ([T.Detn(!line,!line)],yytext);   
("des" | "der" | "des")     
           => ([T.Detg(!line,!line)],yytext);   
("dem" | "der" | "dem")   
           => ([T.Detd(!line,!line)],yytext);   
("den" | "die" | "das")   
           => ([T.Deta(!line,!line)],yytext);
("Ein" | "Eine" | "ein" | "eine") 
        => ([T.EQn(!line,!line)],yytext); 
("Eines" | "Einer" | "eines" | "einer") 
        => ([T.EQg(!line,!line)],yytext); 
("Einem" | "Einer" | "einem" | "einer") 
        => ([T.EQd(!line,!line)],yytext);
("Einen" | "Eine" | "Ein" |"einen" | "eine" | "ein") 
        => ([T.EQa(!line,!line)],yytext);
("Kein" | "Keine" | "kein" | "keine") 
        => ([T.NQn(!line,!line)],yytext);
("Keines" | "Keiner" | "keines" | "keiner") 
        => ([T.NQg(!line,!line)],yytext);  
("Keinem" | "Keiner" | "keinem" | "keiner") 
        => ([T.NQd(!line,!line)],yytext); 
("Keinen" | "Keine" | "Kein" | "keinen" | "keine" | "kein") 
        => ([T.NQa(!line,!line)],yytext); 
("Alle" | "alle" | "Jeder" | "Jede" | "Jedes" | "jeder" | "jede" | "jedes") 
        => ([T.UQn(!line,!line)],yytext);
("Aller" | "aller" | "Jedes" | "Jeder" | "jedes" | "jeder")
        => ([T.UQg(!line,!line)],yytext); 
("Allen" | "allen" | "Jedem" | "Jeder" | "jedem" | "jeder")
        => ([T.UQd(!line,!line)],yytext);
("Alle" | "alle" | "Jeden" | "Jede" | "Jedes" | "jeder" | "jede" | "jedes")
        => ([T.UQa(!line,!line)],yytext);
"Anna"     => (PN "Anna");
"Dieter"   => (PN "Dieter");
"Emil"     => (PN "Emil");
"Maria"    => (PN "Maria");
"Rom"   => (PN "Rom");
"Madrid"   => (PN "Madrid");
"Italien"   => (PN "Italien");
"Spanien"   => (PN "Spanien");
"Mann"     => (CNn "Mann" yytext);
"Mannes"   => (CNg "Mann" yytext);
"Mann"     => (CNd "Mann" yytext);
"Mann"     => (CNa "Mann" yytext);
"Frau"     => (CNn "Frau" yytext);
"Frau"     => (CNg "Frau" yytext);
"Frau"     => (CNd "Frau" yytext);
"Frau"     => (CNa "Frau" yytext);
"Hund"     => (CNn "Hund" yytext);
"Hundes"     => (CNg "Hund" yytext);
"Hund"     => (CNd "Hund" yytext);
"Hund"     => (CNa "Hund" yytext);
"Student"   => (CNn "Student" yytext);
"Studenten" => (CNg "Student" yytext);
"Studenten" => (CNd "Student" yytext);
"Studenten" => (CNa "Student" yytext);
"Bruder"   => (RNn "Bruder" yytext);   
"Bruders"   => (RNg "Bruder" yytext); 
"Bruder"   => (RNd "Bruder" yytext);   
"Bruder"   => (RNa "Bruder" yytext);   
"Schwester"  => (RNn "Schwester" yytext); 
"Schwester"  => (RNg "Schwester" yytext); 
"Schwester"  => (RNd "Schwester" yytext); 
"Schwester"  => (RNa "Schwester" yytext);
"Nachbar"  => (RNn "Nachbar" yytext);   
"Nachbars"  => (RNg "Nachbar" yytext);   
"Nachbarn"  => (RNd "Nachbar" yytext);   
"Nachbarn"  => (RNa "Nachbar" yytext);   
"Hauptstadt"   => (RNn "Hauptstadt" yytext);
"Hauptstadt"   => (RNg "Hauptstadt" yytext);
"Hauptstadt"   => (RNd "Hauptstadt" yytext);
"Hauptstadt"   => (RNa "Hauptstadt" yytext);   
"gesund"   => (CA "gesund");
("in") => (P "in");
"arbeiten" => (IVp "arbeiten");
"arbeitet" => (IVs "arbeiten");
"kennen"     => (TVp "kennen");
"kennt"      => (TVs "kennen");
"lieben"     => (TVp "lieben"); 
"liebt"      => (TVs "lieben");
"tanzen"     => (TVp "tanzen"); 
"tanzt"      => (TVs "tanzen");
.          => (error ("ignoring illegal character " ^ yytext,
                      !line,!line); lex());


